#ifndef __PLOT_UTILITIES__
#define __PLOT_UTILITIES__

/*! \file PlotUtilities.h
 *  \brief Header File for Plot Utility Routines
 */

#include "VectorUtilities.h"
#include "Voronoi.h"

//! Plotting the Assignment Probabilities
/*! \param FileName The Output File Name of Plotted Chart
 *  \param ap The Assignment Probabilities of Individuals
 *  \return The Result of Action (0: Success; -1: Failure)
 *  \author Chibiao Chen
 */
int PlotAP(const char* FileName, const DBL_VCTR_2D& ap);

//! Plotting the Likelihood History
/*! \param FileName The Output File Name of Plotted Chart
 *  \param lh The Likelihood History
 *  \return The Result of Action (0: Success; -1: Failure)
 *  \author Chibiao Chen
 */
int PlotLH(const char* FileName, const DBL_VCTR_1D& lh);

//! Plotting the Hard Clustering Result Using a Scatter Chart (Not In Use Currently)
/*! \param FileName The Output File Name of Plotted Chart
 *  \param vptX The Coordinates of Individuals
 *  \param vdDataScale The Scales of X, Y Coordinate Data
 *  \param hc The Hard Clustering Results (Cluster Labels)
 *  \param NumOfC Number of Clusters
 *  \param bDrawLabel Drawing the Individual Labels?
 *  \return The Result of Action (0: Success; -1: Failure)
 *  \author Chibiao Chen
 */
int PlotHC(const char* FileName, 
		   const vector< SPoint >& vptX,
		   const vector< double >& vdDataScale, 
		   const INT_VCTR_1D& hc, 
		   const int NumOfC, 
		   bool bDrawLabel = false);

//! Plotting the Hard Clustering Result (Color Tessellation & Black Dots)
/*! \param FileName The Output File Name of Plotted Chart
 *  \param vptX The Coordinates of Individuals
 *  \param vbKeep The Individuals Kept in the Chart
 *  \param vdDataMinMax The Limits of X, Y Coordinate Data
 *  \param veVoronoi The Generated Voronoi Edges
 *  \param hc The Hard Clustering Results (Cluster Labels)
 *  \return The Result of Action (0: Success; -1: Failure)
 *  \author Chibiao Chen
 */
int PlotHC(const char* FileName, 
		   const vector< SPoint >& vptX,
		   const vector< bool >& vbKeep, 
		   const vector< double >& vdDataMinMax, 
		   vector< CEdge >& veVoronoi, 
		   const INT_VCTR_1D& hc);

#endif//__PLOT_UTILITIES__
